from typing import Dict, List, Tuple, Callable


Key = List[Tuple[type, Callable]]
Ignore = List[str]


# TODO What if a key only exists in one of given dicts?
def compare_dicts(a: Dict, b: Dict,
                  custom_comp: Key = [], ingore: Ignore = []) -> bool:

    if type(a) != type(b):
        return False

    if isinstance(a, dict):
        if not sorted(a.keys()) == sorted(b.keys()):
            return False
        return all(
            [compare_dicts(a[key], b[key], custom_comp) for key in a.keys()]
        )

    if isinstance(a, list):
        if len(a) != len(b):
            return False
        return all(
            [
                compare_dicts(a_element, b_element, custom_comp)
                for a_element, b_element in zip(a, b)
            ]
        )

    if type(a) in custom_comp:
        return custom_comp[type(a)](a, b)
    return a == b



from chabie import chabie


def test_compare_dicts():
    assert chabie.compare_dicts({}, {})
    assert chabie.compare_dicts({'a': ''}, {'a': ''})
    assert not chabie.compare_dicts({'a': ''}, {'b': ''})
    assert not chabie.compare_dicts({'a': ''}, {})
    assert not chabie.compare_dicts({}, {'b': ''})
    assert not chabie.compare_dicts({'a': ''}, None)
    assert not chabie.compare_dicts(None, {'a': ''})
    assert chabie.compare_dicts(
        {
            'int': 0,
            'bool': False,
            'None': None,
            'empty_str': '',
            'empty_list': [],
            'empty_dict': {}
        },
        {
            'int': 0,
            'bool': False,
            'None': None,
            'empty_str': '',
            'empty_list': [],
            'empty_dict': {}
        },
    )
    assert chabie.compare_dicts(
        {
            'nested_list_1': [1, 2, 3],
            'nested_list_2': [
                {'dict_in_list_1': True},
                {'dict_in_list_2': 'foo'},
            ],
            'nested_dict_1': {'key_1': 'value_1'},
            'nested_dict_2': [
                'yi', 'er', 'san'
            ],
        },
        {
            'nested_list_1': [1, 2, 3],
            'nested_list_2': [
                {'dict_in_list_1': True},
                {'dict_in_list_2': 'foo'},
            ],
            'nested_dict_1': {'key_1': 'value_1'},
            'nested_dict_2': [
                'yi', 'er', 'san'
            ],
        }
    )
    assert not chabie.compare_dicts(
        {
            'nested_list_1': [1, 2, 3],
            'nested_list_2': [
                {'dict_in_list_1': True},
                {'dict_in_list_2': 'foo'},
            ],
            'nested_dict_1': {'key_1': 'value_1'},
            'nested_dict_2': [
                'yi', 'er', 'san'
            ],
            'This key not in the second dict': 'So returns false'
        },
        {
            'nested_list_1': [1, 2, 3],
            'nested_list_2': [
                {'dict_in_list_1': True},
                {'dict_in_list_2': 'foo'},
            ],
            'nested_dict_1': {'key_1': 'value_1'},
            'nested_dict_2': [
                'yi', 'er', 'san'
            ],
        }
    )



from setuptools import setup, find_packages


setup(
    name='chabie',
    version='0.2.1',
    description='Comparison utility',
    author='TatchNicolas',
    author_email='TatchNicolas@users.noreply.github.com',
    license='MIT',
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)
